% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% %                                                                       %
% %    Matlab script for homework of lecture                              %
% %    "Matrix calculus – an introduction using Matlab"                   %
% %    by Christian Rummel                                                %
% %                                                                       %
% %    BENESCO lecture series                                             %
% %    "Signal Analysis and Brain Oscillations in Health and Disease"     %
% %    Bern, 2017/09/29                                                   %
% %                                                                       %
% %    Script demonstrates runtime difference with and without code       %
% %    in vectorization Matlab at the example of matrix thresholding.     %
% %                                                                       %
% %    written by C. Rummel, 2017/09/25                                   %
% %                                                                       %
% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear all
close all
clc

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

N      =   20;

dim1   = 5000;
dim2   = 2000;

thresh =    1;

% loop over independent realizations

runtime = zeros(N,4);

for n = 1:N
    waitbar(n/N);

% 1) generate random matrix

    A = randn(dim1,dim2);

% 2) use loops for original matrix

    A1_org = A;

    t_start_lo = tic;
    for i1 = 1:dim1
        for i2 = 1:dim2
            if abs(A1_org(i1,i2))<thresh
                A1_org(i1,i2) = 0;
            end
        end
    end
    runtime(n,1) = toc(t_start_lo);

% 3) avoid loops for original matrix

    A2_org = A;

    t_start_vo = tic;
    A2_org(abs(A2_org)<thresh) = 0;
    runtime(n,3) = toc(t_start_vo);

% 4) use loops for transposed matrix

    A1_trn = A';

    t_start_lt = tic;
    for i1 = 1:dim2
        for i2 = 1:dim1
            if abs(A1_trn(i1,i2))<thresh
                A1_trn(i1,i2) = 0;
            end
        end
    end
    runtime(n,2) = toc(t_start_lt);

% 5) avoid loops for transposed matrix

    A2_trn = A;

    t_start_vt = tic;
    A2_trn(abs(A2_trn)<thresh) = 0;
    runtime(n,4) = toc(t_start_vt);
    
end

[mean(runtime); std(runtime)]

% a) pairwise t-test for differences between looped and vectorized versions

[h13,p13] = ttest(runtime(:,1),runtime(:,3))
[h24,p24] = ttest(runtime(:,2),runtime(:,4))

% b) pairwise t-test for differences between original and transposed versions

[h12,p12] = ttest(runtime(:,1),runtime(:,2))
[h34,p34] = ttest(runtime(:,3),runtime(:,4))




